from .blacklist import BlacklistToken
from ..api import database as db, flask_bcrypt, config


class Auth(db.Model):
    __tablename__ = "auth"

    aid = db.Column(db.Integer, primary_key=True, autoincrement=True)
    email = db.Column(db.String(255), unique=True, nullable=False)
    passsword_hash = db.Column(db.String(255), nullable=True )

    user_type = db.Column(db.String(255), nullable=True)

    @property
    def password(self):
        raise AttributeError('password: write-only field')

    @password.setter
    def password(self, password):
        self.password_hash = flask_bcrypt.generate_password_hash(password).decode('utf-8')

    def check_password(self, password):
        return flask_bcrypt.check_password_hash(self.password_hash, password)

    @staticmethod
    def login(email, password, user_type=None):
        try:
            user = Auth.query.filter_by(email=data.get('email')).first()
            if user and user.check_password(data.get('password')):
                return True
            else:
                return False
        except Exception as e:
            return None



from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_bcrypt import Bcrypt
from dotenv import load_dotenv
from flask_cors import CORS
#from flask_rest_paginate import Pagination
import importlib
from flask_restplus import Api as ApiRestplus
#from flask import Blueprint
from jsonschema import FormatChecker

load_dotenv()
database = SQLAlchemy()
flask_bcrypt = None
config = {}

class Api():
    app = None
    db = None
    api = None
    flask_bcrypt = None
    app_config=None

    def __init__(self, name, app_config={}, urls=[]):
        #super(ApiRestplus, self).__init__()
        
        self.app = Flask(name, template_folder='templates')
        self.db = database
        self.flask_bcrypt = Bcrypt()
        self.app_config = app_config
        self.api = ApiRestplus(self.app,
            doc=False, 
            specs=False,
            security='Bearer Auth',
            #authorizations=AUTHORIZATIONS,
            title='FLASK RESTPLUS API BOILER-PLATE WITH JWT',
            version='1.0',
            description='a boilerplate for flask restplus web service',
            format_checker=FormatChecker(formats=["date"])
        )
        for mod in urls:
            #full_module_name = "app.controller." + mod["controller"]
            #print(full_module_name)
            #mymodule = importlib.import_module(full_module_name)
            #print(mymodule)
            self.api.add_namespace(mod["controller"].api,path=mod["path"])

    
    def create_app(self, config_name="dev"):
        self.app.config['Upload_folder'] = './static/'
        CORS(self.app, supports_credentials=True, 
            #origins=["http://localhost:3000", "https://www.micasaldm.mx","http://192.168.0.12:3000","https://tec.apoyoestudiantil.mx", "http://tec.apoyoestudiantil.mx" ]
        )
        config = self.app_config[config_name]
        self.app.config.from_object(self.app_config[config_name])

        self.db.init_app(self.app)
        self.flask_bcrypt.init_app(self.app)

        @self.app.teardown_appcontext
        def shutdown_session(exception=None):
            self.db.session.close()

        database = self.db
        
        flask_bcrypt = self.flask_bcrypt
        return self.app
    
    def get_api_db(self):
        return self.db



"""
app = Flask(__name__, template_folder='templates')
db = SQLAlchemy()
flask_bcrypt = Bcrypt()
pagination = Pagination(app, db)
blueprint = Blueprint('api', __name__, url_prefix="")

CONTROLLERS = [
    {"controller":"chatbot", "path":"/chatbot"}
]


api = Api(blueprint,
    doc=False, specs=False,
    security='Bearer Auth',
    authorizations=AUTHORIZATIONS,
    title='FLASK RESTPLUS API BOILER-PLATE WITH JWT',
    version='1.0',
    description='a boilerplate for flask restplus web service',
    format_checker=FormatChecker(formats=["date"])
)


for mod in CONTROLLERS:
    full_module_name = "app.main.controller." + mod["controller"]
    print(full_module_name)
    mymodule = importlib.import_module(full_module_name)
    print(mymodule)
    api.add_namespace(mymodule.api,path= mod["path"])



def create_app(config_name):
    app.config['Upload_folder'] = './static/'
    CORS(app, supports_credentials=True, origins=["http://localhost:3000", "https://www.micasaldm.mx","http://192.168.0.12:3000","https://tec.apoyoestudiantil.mx", "http://tec.apoyoestudiantil.mx" ])
    app.config.from_object(config_by_name[config_name])
    db.init_app(app)
    flask_bcrypt.init_app(app)
    #session  = Session(app)
    #app.config["SESSION_SQLALCHEMY"] = db
    #session.app.session_interface.db.create_all()

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.close()

    return app
"""


import os
from flask_migrate import Migrate, MigrateCommand
from flask_script import Manager
from .api import database as db

application = None
migrate = None

class ApiManager():
    def __init__(self, app):
        application = app
        application.app_context().push()
        manager = Manager(application)

        migrate = Migrate(application, db, directory="./database/migrations")
        manager.add_command('db', MigrateCommand)

        with application.app_context():
            from .models import imports

        @manager.command
        def start():
            application.run(host="0.0.0.0")
        
        manager.run()


from ..api import database  as db
import datetime
from ..database import  TimestampMixin

class BlacklistToken(db.Model, TimestampMixin):
    """
    Token Model for storing JWT tokens
    """
    __tablename__ = 'blacklist_tokens'

    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    token = db.Column(db.String(500), unique=True, nullable=False)
    blacklisted_on = db.Column(db.DateTime, nullable=False)

    def __init__(self, token):
        self.token = token
        self.blacklisted_on = datetime.datetime.now()

    def __repr__(self):
        return '<id: token: {}'.format(self.token)

    @staticmethod
    def check_blacklist(auth_token):
        # check whether auth token has been blacklisted
        res = BlacklistToken.query.filter_by(token=str(auth_token)).first()
        if res:
            return True
        else:
            return False


import datetime
from sqlalchemy.orm import MapperExtension, Query
from sqlalchemy.sql import null
from flask_sqlalchemy import BaseQuery, Model
from sqlalchemy.orm import relationship as rs
from sqlalchemy import event
from sqlalchemy import inspect
import sqlalchemy as sa


class FillableMixin:
    fillable = []
    def fill(self, data):
        for key in self.fillable:
            if key in data:
                setattr(self, key, data[key])
                #print(key + " " + data[key])  

class TsMapper(MapperExtension):
    def before_update(self, mapper, connection, instance):
        """ Make sure when we update this record the created fields stay unchanged!  """
        instance.updated_at = datetime.datetime.utcnow()

class TimestampMixin(MapperExtension):
    created_at = sa.Column(sa.DateTime(), default=datetime.datetime.utcnow())
    updated_at = sa.Column(sa.DateTime(), onupdate=datetime.datetime.now)

    __mapper_args__ = { 'extension': TsMapper() }  


class SoftDeleteMixin:
    deleted_at = sa.Column(sa.DateTime(timezone=True), nullable=True)

    def delete(self, deleted_at=None):
        self.deleted_at = deleted_at or datetime.datetime.utcnow()

    def restore(self):
        self.deleted_at = None
 
@event.listens_for(Query, 'before_compile', retval=True)
def before_compile(query):
    #print(str(query))
    include_deleted = query._execution_options.get('include_deleted', False)
    if include_deleted:
        return query

    for column in query.column_descriptions:
        entity = column['entity']
        if entity is None:
            continue

        inspector = inspect(column['entity'])
        mapper = getattr(inspector, 'mapper', None)
        if mapper and issubclass(mapper.class_, SoftDeleteMixin):
            query = query.enable_assertions(False).filter(
                entity.deleted_at.is_(None),
            )
        
    return query


@event.listens_for(SoftDeleteMixin, 'load', propagate=True)
def load(obj, context):
    include_deleted = context.query._execution_options.get('include_deleted', False)
    if obj.deleted_at and not include_deleted:
        raise TypeError('Deleted object '+str(obj)+' was loaded, did you use joined eager loading?')
    

class TimestampMixin(MapperExtension):
    created_at = sa.Column(sa.DateTime(), default=datetime.datetime.utcnow())
    updated_at = sa.Column(sa.DateTime(), onupdate=datetime.datetime.now)

    __mapper_args__ = { 'extension': TsMapper() }  


from .. import database as db, flask_bcrypt, config
import pickle
import uuid
import datetime
import jwt
from itsdangerous import want_bytes


class Session(db.Model):
    __tablename__ = "session"

    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    session_id = db.Column(db.String(255), unique=True)
    data = db.Column(db.LargeBinary)
    expiry = db.Column(db.DateTime)

    data_ = {}

    def get(self, key):
        return self.data_.get(key,False)

    def set(self, key, val):
        self.data_[key] = val

    def __repr__(self):
        return '<Session data %s>' % self.session_id

    @staticmethod
    def open(auth_token=None):
        if not auth_token:
            sid = str(uuid.uuid4())
            new_session = Session(session_id=sid, expiry=datetime.datetime.utcnow() + datetime.timedelta(days=1, seconds=5))
            val = pickle.dumps(dict())
            new_session.data = val
            db.session.add(new_session)
            db.session.commit()
            auth_token = Session.encode_auth_token(sid=sid)
            new_session.data_["auth_token"] = auth_token
            return new_session
        else:
            sid = Session.decode_auth_token(auth_token)
            if sid:
                saved_session = Session.query.filter_by(session_id=sid).first()
                if saved_session and saved_session.expiry <= datetime.datetime.utcnow():
                    db.session.delete(saved_session)
                    db.session.commit()
                    saved_session = None

                if saved_session:
                    val = saved_session.data
                    saved_session.data_ = pickle.loads(want_bytes(val))
                    return saved_session
            return None

    @staticmethod
    def delete(auth_token):
        sid = Session.decode_auth_token(auth_token)
        saved_session = Session.query.filter_by(session_id=sid).first()
        if saved_session:
            db.session.delete(saved_session)
            db.session.commit()


    def save(self):
        val = pickle.dumps(dict(self.data_))
        self.data = val
        db.session.add(self)
        db.session.commit()


    @staticmethod 
    def encode_auth_token(sid):
        try:
            payload = {'sid': sid}
            token = jwt.encode(
                payload,
                config["key"],
                algorithm='HS256'
            ).decode('utf-8')
            return token
        except Exception as e:
            return e
        
    @staticmethod  
    def decode_auth_token(auth_token):
        try:
            payload = jwt.decode(auth_token, config["key"], algorithms=['HS256'])
            return payload["sid"]
        except Exception as e:
            return False




from setuptools import setup

setup(name='flaskapitools',
      version='0.1.65',
      #description='The funniest joke in the world',
      #url='http://github.com/storborg/funniest',
      author='Jorge Mendez',
      author_email='jolu.mm@hotmail.com',
      license='MIT',
      packages=['flaskapitools', 'flaskapitools.models'],
      install_requires=[
            "Flask >= 1.1.2",
            "Flask-SQLAlchemy",
            "PyMySQL >= 0.9.3",
            "SQLAlchemy >= 1.3.16",
            "Flask-Bcrypt==0.7.1",
            "Flask-Cors==3.0.8",
            "flask-restplus==0.13.0",
            "Werkzeug==0.16.1",
            "python-dotenv",
            "Flask-Script==2.0.6",
            "PyJWT",
            "Flask-Migrate==2.5.3"
      ],
      zip_safe=False
)